%===================================================================%
% This file simulates the Small Scale NK model in
% Aruoba, Cuba-Borda, Schorfheide (2016)
%
% Model: discount factor shocks and output growth rule
%
% This version: August 2, 2016
%===================================================================%


%===================================================================%
%                    DECLARATION OF VARIABLES                       %
%===================================================================%

var

% ENDOGENOUS VARIABLES
R          % (1)  Federal Fund Rate (FFR)
g          % (2)  Public Spending
z          % (3)  TFP
d          % (4)  Preference shock
y          % (5)  GDP
c          % (6)  Consumption
infl;      % (7)  Inflation

% EXOGENOUS SHOCKS
varexo e_z e_g e_d e_r;

%=========================================================================%
%                       DECLARATION OF PARAMETERS                         %
%=========================================================================%

parameters

% Free parameters
tau         % (1)  Inverse of IES
kappa       % (2)  Slope of Phillips Curve
psi_1       % (3) Sensitivity of FFR to Inflation
psi_2       % (4) Sensitivity of FFR to GDP growth
rho_r       % (5)  Interest Rate Smoothing parameter
rho_g       % (6)  Persistence, Government Spending
rho_z       % (7)  Persistence, TFP growth
rho_d       % (8)  Persistence, discount factor shock
sigma_r     % (9)  Volatility, Monetary Policy Shock
sigma_g     % (10)  Volatility, Government Spending
sigma_z     % (11)  Volatility, TFP growth
sigma_d     % (12) Volatility, preference shock

% Fixed parameters
R_star_an   % (1) Interest Rate Target
pi_star_an  % (2) Inflation Target
gamma_qu    % (3) Growth Rate of Output (quarterly)
cy          % (4) C/Y ratio
g_star      % (5) Steady state level of government expenditure
nu          % (6) EOS intermediate goods (markup)
eta;        % (7) Frisch elasticity


%=========================================================================%
%                        PARAMETER VALUES                                 %
%=========================================================================%

% Parameters to be Estimate
set_param_value('tau',params0(1))
set_param_value('kappa',params0(2))
set_param_value('psi_1',params0(3))
set_param_value('psi_2',params0(4))
set_param_value('rho_r',params0(5))
set_param_value('rho_g',params0(6))
set_param_value('rho_z',params0(7))
set_param_value('rho_d',params0(8))
set_param_value('sigma_r',params0(9))
set_param_value('sigma_g',params0(10))
set_param_value('sigma_z',params0(11))
set_param_value('sigma_d',params0(12))

% Fixed Parameters
set_param_value('R_star_an',paramsfixed(1))
set_param_value('pi_star_an',paramsfixed(2))
set_param_value('gamma_qu',paramsfixed(3))
set_param_value('cy',paramsfixed(4))
set_param_value('g_star',paramsfixed(5))
set_param_value('nu',paramsfixed(6))
set_param_value('eta',paramsfixed(7))


%=========================================================================%
%                        EQUILIBRIUM CONDITIONS                           %
%=========================================================================%
model;

%=========================IMPLICITELY DEFINED PARAMETERS==================%
#pi_star = 1+(pi_star_an/400);
#gamma   = 1+(gamma_qu/100);
#bet     = 400/(400+R_star_an);
#phi     = tau*(1-nu)/(nu*(pi_star^(2))*kappa);
#pibar   = pi_star;
#b       = 1-(1/(2*nu));
#chi_h   = 1;

%========================= STEADY STATE (EXPRESSED IN LOGS) ==============%

#yss = ((1 - nu)*g_star^tau)^(1/(tau + 1/eta));

#css = yss / g_star;

#cst = log(css);

#yst = log(yss);

#pst = log(pi_star);

#gst = log(g_star);


%=========================ENDOGENOUS VARIABLES============================%

%(1)
1 = exp(-tau*(c(+1)-c) + d(+1) - d - z(+1) + R - infl(+1));

%(2)
1 = (1/nu)*(1-exp(tau*(c+cst) + (1/eta)*(y+yst)))
  + phi*(exp(infl+pst)-pibar)*(b*exp(infl+pst)+(1-b)*pibar)
  - phi*bet*(exp(-tau*(c(+1)-c) + y(+1) - y + d(+1) - d + infl(+1)+ pst)*(exp(infl(+1)+pst) - pibar));

%(3)
exp(c+cst-y-yst) = exp(-g-gst) - (phi/2)*(exp(infl+pst)-pibar)^2;

%(4)
exp(R) = exp(rho_r*R(-1) + (1-rho_r)*psi_1*infl + (1-rho_r)*psi_2*(y - y(-1) + z)+ sigma_r*e_r);

%(5)
g = rho_g*g(-1) + sigma_g*e_g;

%(6)
z = rho_z*z(-1) + sigma_z*e_z;

%(7)
d = rho_d*d(-1) + sigma_d*e_d;

end;

%===================================================================%
%                INITIAL CONDITIONS FOR STEADY STATE                %
%===================================================================%

steady_state_model;
c        = 0;
y        = 0;
infl     = 0;
g        = 0;
z        = 0;
d        = 0;
R        = 0;
end;

%===================================================================%
%              DEFINE STDEV OF STRUCTURAL INNOVATIONS               %
%===================================================================%
shocks;
var e_z;
stderr 1;
var e_g;
stderr 1;
var e_d;
stderr 1;
var e_r;
stderr 1;
end;

% Activate the following three lines to check model solution
steady;
check;
stoch_simul(order=1,periods=200,irf=0,nograph);

% Move files to folder
% move ACS_OutputGrowth_loglinear